#!/bin/sh

####### usage #####
#
# debug=1 rps_cpus=f rps_flow_entries=1024 /usr/sbin/set_throughput_rule.sh br-lan
#
###################

get_cpu_num() {
	local cpunum=`cat /proc/cpuinfo |grep ^processor |wc -l`
	echo ${cpunum:-1}
}

get_cpu_mask() {
	local cpunum=$(get_cpu_num)
	local cpumask=0
	while [ $cpunum -gt 1 ]
	do
		cpunum=$(($cpunum-1))
		cpumask=$(($cpumask+(1 << $cpunum)))
		# remain cpu0 for system use
		[ $cpunum -le 0 ] && break;
	done
	printf %x $cpumask
}

get_wifi_vaps() {
	local wifi_device=$1
	local all_vaps=`ls /sys/class/net/**/parent`
	local vaps=""
	for vap in $all_vaps
	do
		local parent=$(cat $vap)
		local path=$(dirname $vap)
		if [ "$parent" = "$wifi_device" ]
		then
			vaps="$vaps${vaps:+ }${path##*/}"
		fi
	done
	echo "$vaps"
}

set_throughput_rule() {
	local cpu_num=$(get_cpu_num)
	local ifaceList="$@"

	if [ $cpu_num -gt 1 -a -f /proc/sys/net/core/rps_sock_flow_entries ]
	then
		if [ "$ifaceList" != "" ]
		then
			for iface in $ifaceList
			do
				if [ ! -d "/sys/class/net/$iface" ]
				then
					echo "$iface not found!"
				fi
			done
		else
			vapList_24G=$(get_wifi_vaps wifi0)
			vapList_5G=$(get_wifi_vaps wifi1)
			vapList=$vapList_24G" "$vapList_5G
			if [ $cpu_num -eq 2 ]
			then
				ifaceList="is_24g_iface "$vapList_24G" is_5g_iface "$vapList_5G
			else
				ifaceList=$vapList
			fi
		fi

		local rps_sock_flowentries=`cat /proc/sys/net/core/rps_sock_flow_entries`
		if [ "$rps_sock_flowentries" = "0" ]
		then
			# linux document recommend set to 32768
			rps_sock_flowentries=32768
			echo $rps_sock_flowentries > /proc/sys/net/core/rps_sock_flow_entries
			[ ${debug:-0} -eq 1 ] && echo "echo $rps_sock_flow_entries > /proc/sys/net/core/rps_sock_flow_entries"
		fi

		if [ "$rps_cpus" = "" ]
		then
			local cpumask=$(get_cpu_mask)
		else
			local cpumask=$rps_cpus
		fi

		if [ "$rps_flow_entries" = "" ]
		then
			local rps_flowentries=$rps_sock_flowentries
		else
			local rps_flowentries=$rps_flow_entries
		fi

		for iface in $ifaceList
		do
			if [ $cpu_num -eq 2 ]
			then
				if [ "$iface" = "is_24g_iface" ]
				then
					cpumask=1
				elif [ "$iface" = "is_5g_iface" ]
                then
					cpumask=2
				fi
			fi

			local rps_queuenum=`ls /sys/class/net/$iface/queues/ |grep "rx-*"| wc -l`
			local rps_flowcnt=`echo $(($rps_flowentries/${rps_queuenum:-1}))`

			for rxqueue in `find /sys/class/net/$iface/queues/ -name "rx-*"`
			do
				echo $cpumask > $rxqueue/rps_cpus
				echo $rps_flowcnt > $rxqueue/rps_flow_cnt
				[ ${debug:-0} -eq 1 ] && {
					echo "echo $cpumask > $rxqueue/rps_cpus"
					echo "echo $rps_flowcnt > $rxqueue/rps_flow_cnt"
				}
			done
		done
	fi
}

set_throughput_rule "$@"

